# a modified version of https://github.com/khuyentran1401/rich-dataframe/blob/master/rich_dataframe/rich_dataframe.py

import time
from contextlib import contextmanager

import pandas as pd
from rich import print
from rich.box import MINIMAL, SIMPLE, SIMPLE_HEAD, SQUARE
from rich.columns import Columns
from rich.console import Console
from rich.live import Live
from rich.measure import Measurement
from rich.table import Table

console = Console()

BEAT_TIME = 0.008

COLORS = ["cyan", "magenta", "red", "green", "blue", "purple"]


@contextmanager
def beat(length: int = 1) -> None:
    with console:
        yield
    time.sleep(length * BEAT_TIME)


class DataFramePrettify:
    def __init__(
        self,
        df: pd.DataFrame,
        delay_time: int = 5,
        clear_console: bool = True,
    ) -> None:
        self.df = df.reset_index().rename(columns={"index": ""})
        self.df["Country"] = (self.df.index + 1).astype(str) + " " + self.df["Country"].astype(str)
        self.table = Table(show_footer=False)
        self.table_centered = Columns((self.table,), align="center", expand=True)
        self.num_colors = len(COLORS)
        self.delay_time = delay_time
        self.clear_console = clear_console

        self.columns = self.df.columns
        self.rows = self.df.values

        if self.clear_console:
            console.clear()

    def _add_columns(self):
        for col in self.columns[1:]:
            with beat(self.delay_time):
                self.table.add_column(str(col))

    def _add_rows(self):
        for row in self.rows:
            with beat(self.delay_time):
                row = [str(item) for item in row[1:]]
                self.table.add_row(*list(row))

    def _add_random_color(self):
        for i in range(len(self.table.columns)):
            with beat(self.delay_time):
                self.table.columns[i].header_style = COLORS[i % self.num_colors]

    def _add_style(self):
        for i in range(len(self.table.columns)):
            with beat(self.delay_time):
                self.table.columns[i].style = "bold " + COLORS[i % self.num_colors]

    def _adjust_box(self):
        for box in [SIMPLE_HEAD, SIMPLE, MINIMAL, SQUARE]:
            with beat(self.delay_time):
                self.table.box = box

    def _dim_row(self):
        with beat(self.delay_time):
            self.table.row_styles = ["none", "dim"]

    def _adjust_border_color(self):
        with beat(self.delay_time):
            self.table.border_style = "bright_yellow"

    def _change_width(self):
        original_width = Measurement.get(console, self.table).maximum
        width_ranges = [
            [original_width, console.width, 2],
            [console.width, original_width, -2],
            [original_width, 90, -2],
            [90, original_width + 1, 2],
        ]

        for width_range in width_ranges:
            for width in range(*width_range):
                with beat(self.delay_time):
                    self.table.width = width

            with beat(self.delay_time):
                self.table.width = None

    def _add_caption(self):
        with beat(self.delay_time):
            self.table.caption = (
                f"All [bold green]{len(self.rows)}[/bold green] found positions are shown here."
            )

    def prettify(self):
        with Live(
            self.table_centered,
            console=console,
            refresh_per_second=self.delay_time,
            vertical_overflow="ellipsis",
        ):
            self._add_columns()
            self._add_rows()
            self._add_random_color()
            self._add_style()
            self._adjust_border_color()
            # self._add_caption()

        return self.table


def prettify(
    df: pd.DataFrame,
    delay_time: int = 5,
    clear_console: bool = True,
):
    """Create animated and pretty Pandas DataFrame
    Parameters
    ----------
    df : pd.DataFrame
        The data you want to prettify
    delay_time : int, optional
        How fast is the animation, by default 5. Increase this to have slower animation.
    clear_console: bool, optional
        Clear the console before priting the table, by default True. If this is set to false the previous console input/output is maintained
    """
    if isinstance(df, pd.DataFrame):
        DataFramePrettify(df, delay_time, clear_console).prettify()

    else:
        # In case users accidentally pass a non-datafame input, use rich's print instead
        print(df)



#!/usr/bin/env python3

import asyncio
import re
import sys
from pathlib import Path
from datetime import date
from dataclasses import dataclass
from typing import Optional, List

import httpx
import pandas as pd
import rich
import urllib3
from bs4 import BeautifulSoup as bs
from rich.console import Console

sys.path.append(
    Path(__file__).parent.parent.as_posix()
)  # https://stackoverflow.com/questions/16981921
from phdseeker.rich_dataframe import prettify
from phdseeker.constants import __version__

# Turns off some warnings
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

console = Console()


@dataclass
class Config:
    config = {
        "scholarshipdb": {
            "sought#": "h1.title",
            "query": "https://scholarshipdb.net/scholarships/Program-PhD?page={page}&q={fields}",
            "title": "h4 a",
            "country": ".list-unstyled a.text-success",
            "date": ".list-unstyled span.text-muted",
            "link": ".list-unstyled h4 a",
        },
        "findaphd": {
            "sought#": "h4.course-count.d-none.d-md-block.h6.mb-0.mt-1",
            "query": "https://www.findaphd.com/phds/non-eu-students/?01w0&Keywords={fields}&PG={page}",
            "title": "h4 text-dark mx-0 mb-3",
            "country": "country-flag img-responsive phd-result__dept-inst--country-icon",
            "date": "apply py-2 small",
            "link": "h4 text-dark mx-0 mb-3",
        },
    }

    def __init__(self, repo="scholarshipdb"):
        self.repo = repo

    @classmethod
    def repos(cls):
        return ",".join(list(cls.config))

    @property
    def query(self):
        return Config.config[self.repo]["query"]

    @property
    def sought(self):
        return Config.config[self.repo]["sought#"]

    @property
    def title(self):
        return Config.config[self.repo]["title"]

    @property
    def link(self):
        return Config.config[self.repo]["link"]

    @property
    def country(self):
        return Config.config[self.repo]["country"]

    @property
    def date(self):
        return Config.config[self.repo]["date"]

    @property
    def baseURL(self):
        return next(
            re.finditer(r"^.+?[^\/:](?=[?\/]|$)", Config.config[self.repo]["query"])
        ).group()


class PhDSeeker:
    def __init__(
        self,
        keywords: str,
        repos: str = "scholarshipdb, findaphd",
        maxpage: int = 10,
        desired_countries: Optional[str] = None,
    ):
        self.repos = map(str.strip, repos.split(","))  # 'scholarshipdb, findaphd'
        self.keywords = keywords
        self.fields = "%20".join(
            [f'"{item.replace(" ", "%20")}"' for item in map(str.strip, keywords.split(","))]
        )
        self.desired_countries = desired_countries.split(",") if desired_countries else []
        self.desired_countries = [country.strip().lower() for country in self.desired_countries]
        self.titles: List[str] = []
        self.countries: List[str] = []
        self.dates: List[str] = []
        self.links: List[str] = []
        self.maxpage = maxpage
        self.file_name = f"PhD_Positions_{date.today()}[{keywords}]"
        self.df = None  # DataFrame of found positions
        self.sought_number = 0
        self.loop = asyncio.get_event_loop()

    def __str__(self):
        if self.sought_number:
            s = "s" if self.maxpage > 1 else ""
            caption = (
                "│"
                + f"Out of {self.sought_number} found Ph.D. positions, {len(self.df)} have been fetched in {self.maxpage} page{s}.".center(
                    console.width - 2
                )
                + ("│\n└" + "─" * (console.width - 2) + "┘\n")
            )
            prettify(self.df[["Country", "Date", "Title"]], clear_console=False)
            return f"{caption}"

    async def __get_page__(self, repo, page):
        headers = {
            "user-agent": "curl/7.83.0",
            "accept": "*/*",
            "scheme": "http",
            # 'Content-Length': '0',
        }
        c = Config(repo)
        try:
            query = c.query.format(fields=self.fields, page=page)
            async with httpx.AsyncClient(verify=(repo == "findaphd")) as client:
                response = await client.get(query, headers=headers)

            soup = bs(response.text, "html.parser")

            if page == 1:  # get the number of sought positions
                if (n := soup.select_one(c.sought)) is not None:
                    foundPositions = int(re.search("(\d+[,\d*]*)", n.text)[1].replace(",", ""))
                    self.sought_number += foundPositions
                try:
                    sn = f">> {foundPositions} positions found <<"
                except Exception:
                    sn = ">> No positions found <<"
                print(f"\r{sn.center(console.width)}")
            titles, countries, dates, links = [
                soup.select(item) if repo == "scholarshipdb" else soup.find_all(class_=item)
                for item in (c.title, c.country, c.date, c.link)
            ]
            assert titles != [], "No titles found"
            for title, country, date, link in zip(titles, countries, dates, links):
                if repo == "scholarshipdb":
                    found_country = country.text
                else:
                    found_country = country["title"]

                save_position = not self.desired_countries  # all countries
                if found_country and found_country.lower() in self.desired_countries:
                    save_position = True
                if save_position:
                    self.countries.append(found_country)
                    self.titles.append((title.text).strip())
                    self.dates.append(date.text.replace("\n", ""))
                    self.links.append(c.baseURL + link["href"])
            if self.sought_number:
                print(f"\rPage {page} has been fetched from {c.baseURL}!", end="")
            return True
        except AssertionError:
            return False  # break
        except Exception as e:
            # print(e)
            return False  # break

    async def prepare(self):
        for repo in self.repos:
            print(f"\r{('::[ ' + repo + ' ]::').center(console.width, '=')}")
            tasks = [
                asyncio.create_task(self.__get_page__(repo, page))
                for page in range(1, self.maxpage + 1)
            ]
            try:
                await asyncio.wait(tasks, return_when=asyncio.FIRST_EXCEPTION)
            except Exception:
                for t in tasks:
                    t.cancel()

    @property
    def positions(self):
        # asyncio.run(self.prepare())
        try:
            self.loop.run_until_complete(self.prepare())
        finally:
            self.loop.close()
        positions = {
            "Country": self.countries,
            "Date": self.dates,
            "Title": self.titles,
            "Link": self.links,
        }
        self.df = pd.DataFrame.from_dict(positions, orient="index").transpose()
        self.df["timedelta"] = self.df["Date"].apply(lambda x: re.sub("about|ago", "", x).strip())
        name2days = {
            "minutes": "*1",
            "hours": "*60",
            "hour": "*60",
            "days": "*1440",
            "day": "*1440",
            "weeks": "*10080",
            "week": "*10080",
            "months": "*43200",
            "month": "*43200",
        }
        self.df.replace({"timedelta": name2days}, regex=True, inplace=True)
        # eval is not applicable to the empty string
        self.df["timedelta"] = self.df["timedelta"].apply(lambda x: eval(x) if x else x)
        self.df.sort_values(by=["Country", "timedelta", "Title"], inplace=True)
        self.df = self.df.drop("timedelta", axis=1).reset_index(drop=True)
        return self.df

    def save(self, output="both"):
        """Create excel/csv files based on all received data."""
        df = self.positions
        if self.sought_number:
            s = "s" if output == "both" else ""
            print(
                f"\r{console.width * ' '}\n>>>> {self.sought_number} positions have been found in total.",
                f"Specifically, {len(df)} records of them have been saved in the following file{s}:",
                sep="\n",
            )
            if output in ("csv", "both"):
                df.to_csv(f"{self.file_name}.csv", index=False)
                rich.print(f"[blue]{self.file_name}.csv saved![/blue]")
            if output in ("xlsx", "both"):
                df.to_excel(f"{self.file_name}.xlsx", index=False)
                rich.print(f"[blue]{self.file_name}.xlsx saved![/blue]")
        else:
            rich.print("[red blink] >>> No positions found, change your keyword. <<< [/red blink]")


def checkNewVersion(output: dict):
    url = "https://raw.githubusercontent.com/Aghababaei/PhD-Seeker/master/phdseeker/constants.py"
    response = httpx.get(url)
    match = re.search('__version__ = "(\d+\.\d+(?:\.\d+)?)"', response.text)
    if match:
        url_version = match.group(1)
        version = lambda v: list(map(int, v.split(".")))
        if version(url_version) > version(__version__):
            message = (
                "[blink]New version ([green]{}[/green]) is available![/blink]"
                "Use `pip install --upgrade phdseeker` to update"
            )
            output["message"] = message.format(url_version)


def main():
    # Comma separated list of keywords for the field of desired PhD career + presets
    keywords = "Computer Science, Machine Learning, Deep Learning"

    ps = PhDSeeker(keywords, maxpage=10)
    ps.save()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
phdseeker

Usage:
    phdseeker -h
    phdseeker -V
    phdseeker [-k <keywords> -c <countries> --maxpage=<n> --output=<filetype(s)> -v]

options:
    -h --help                       Show this screen.
    -V --version                    Output version information, and repositories' list and exit.
    -v --verbose                    Show the found positions on the terminal.
    -k <keywords>, --keywords=<keywords>    Declare desired keywords to seek. [default: Computer Science, Machine Learning, Deep Learning]
    -c <countries>, --countries=<countries>    Filter by countries.
    -o <filetype(s)>, --output=<filetype(s)>     Set the output type csv/xlsx/both [default: both]
    --maxpage=<n>                   Maximum number of pages to fetch. [default: 10]
"""

import sys
from pathlib import Path
from threading import Thread
from time import perf_counter

import rich
from docopt import docopt

sys.path.append(
    Path(__file__).parent.parent.as_posix(),
)  # https://stackoverflow.com/questions/16981921
from phdseeker.main import PhDSeeker, checkNewVersion, Config
from phdseeker.constants import __version__


def main(args=docopt(__doc__)):
    if args["--version"]:
        rich.print(f"PhD-Seeker Version {__version__}")
        rich.print("\n>> Repositories included <<")
        for i, repo in enumerate(Config.repos().split(","), 1):
            c = Config(repo)
            rich.print(f"{i}. {repo:14}: ", end="")
            rich.print(c.baseURL)
        sys.exit()

    update = {"message": None}
    Thread(target=checkNewVersion, args=(update,), daemon=True).start()

    keywords = args["--keywords"]
    countries = args["--countries"]
    maxpage = int(args["--maxpage"])
    output = args["--output"]

    s = "s" if maxpage > 1 else ""
    incountries = f"in '{countries}' " if countries != None else ""
    rich.print(f"Searching for the Keywords '{keywords}' {incountries}up to {maxpage} page{s}.")
    s = perf_counter()
    ps = PhDSeeker(keywords, maxpage=maxpage, desired_countries=countries)
    ps.save(output)
    rich.print(f"Elapsed time is {perf_counter() - s:.2f}")

    if update["message"]:
        rich.print(update["message"])

    if args["--verbose"] and ps.sought_number:
        rich.print(ps)


if __name__ == "__main__":
    main(docopt(__doc__))



"""setup.py: setuptools control."""

import re
from setuptools import setup


version = re.search(
    r'(__version__ = "(\d\.\d(\.\d+)?)")',
    open("phdseeker/constants.py", encoding="utf8").read(),
    re.M
)[2]

setup(
    name="phdseeker",
    version=version,
    author="Amin Aghababaei, Javad Razavian, Ana Paula Gomes",
    author_email="amin.aghababaei@outlook.com, javadr@gmail.com",
    maintainer='Javad Razavian',
    maintainer_email="javadr@gmail.com",
    packages=['phdseeker',],
    install_requires=[
        'brotlipy',
        'bs4',
        'docopt',
        'httpx',
        'openpyxl',
        'pandas',
        'rich',
        'urllib3',
    ],
    python_requires=">=3.8",
    package_dir={'phdseeker': 'phdseeker'},
    description="PhD Seeker is a python web scraper to search for fully funded doctorate positions, advertised on well-known academic position websites.",
    license="GPLv3",
    keywords="Ph.D. Positions, phdseeker, Academia",
    url="https://github.com/Aghababaei/PhD-Seeker",
    entry_points={
        'console_scripts': [
            'phdseeker = phdseeker.phdseeker_cli:main',
        ],
    },
    long_description_content_type="text/markdown",
    long_description=open("README.md", encoding="utf8").read(),
)
